# Calculates error variance from an observed variance and a reliability.
#  Given the sample size used to estimate the variance and a variance 
#  for the reliability estimate, also gives a variance estimate
#  for the error variance.
get.err.var <- function(obs.var, nobs, rel, var.rel, excess.kurtosis = 0) {

    # Variance of the variance. If no kurtosis is given, normality is 
    #  assumed
    var.obs.var <- obs.var^2 * ((2/(nobs - 1)) + (excess.kurtosis/nobs))

    # Error variance equals (see for example Saris & Gallhofer 2007): 
    err.var <- (1 - rel) * obs.var

    # Variance of the error variance depends on 
    #   1. Sampling variance of the variance of SCF
    #   2. Variance of the reliability estimate rel
    var.err.var.params <- diag(c(var.rel, var.obs.var))

    # First order derivative of the error variance wrt obs. variance & reliability
    deriv.err.var <- rbind(-obs.var, 1 - rel)

    var.err.var <- t(deriv.err.var) %*% var.err.var.params %*% deriv.err.var
    
    return(c('error.variance'=err.var, 
             'var.error.variance'=var.err.var))
}
