library(OpenMxExtras)

data(guillen)
variance.scf <- guillen[1] # The observed variance of the variable SCF

# Sample size for the Guillen et al. data is 223:
nobs <- 223

# Names of the latent and observed variables in the model
latents <- c("Achievement", "Power", "Self-confidence")
manifests <- colnames(guillen)

p <- length(manifests)

# Reliability of SCF as given in the paper
rel.scf <- 0.76

# Standard error of the reliability (calculated from Guillen et al.)
sd.rel.scf <- 0.04

# Define a function to run the above analysis
estimate.model <- function(rel.scf) {
    # Calculate error variance for this replication
    err.var.scf.i <- (1 - rel.scf) * variance.scf

    # Define the OpenMx model with SCF as an observed single indicator
    # of the factor Self-confidence:
    guillenModel <- 
      mxModel(name="Guillen",
              type="RAM",
              manifestVars = manifests,
              latentVars = latents,
              mxPath(from="Achievement", to="INI", free=FALSE, values=1),
              mxPath(from="Achievement",
                     to=c("OPT", "ACH", "CHG")),
              mxPath(from="Power", to="DEV", free=FALSE, values=1),
              mxPath(from="Power",
                     to=c("CONF", "INF", "LEAD")),
              mxPath(from="Self-confidence", to=latents[1:2]),
              mxPath(from="Self-confidence", to="SCF", free=FALSE, values=1),
              mxPath(from=manifests, arrows=2),
              mxPath(from="SCF", arrows=2, free=FALSE, values=err.var.scf.i),
              mxPath(from=latents, arrows=2),
              mxData(observed=round(guillen, 6), type="cov", numObs=223)
              )

     guillenFit <- mxRun(guillenModel)
     
     list(est=guillenFit@output$estimate,
           std.err=guillenFit@output$standardErrors)
}

errfunc <- function(e)  {
     # If no convergence add NA's to the result data frame
     list(est=rep(NA, p), sd.naive=rep(NA, p), sd.correct=rep(NA, p))
}
