\name{get.var.corrected}
\alias{get.var.corrected}
\alias{get.stderr.corrected}
\title{Correct standard errors of a structural equation model for 
    estimation error in fixed parameters.}
\description{
    Whenever a parameter is fixed in the estimation of a model the 
    standard errors of the model estimates are calculated assuming
    that the fixed parameter was known exactly. 
    
    When the fixed parameter was not known exactly but, for example, 
    an estimated statistic from a previous analysis, the standard 
    errors of the free parameters of the model should be corrected.
    
    Given user input, this function provides standard errors for a 
    structural equation model estimated with OpenMx that are 
    corrected for the uncertainty in fixed parameters.
}
\details{
    The function get.stderr.corrected() provides the corrected standard
    errors, while get.var.corrected() gives the full variance-covariance
    matrix.
}
\usage{get.var.corrected(mxFit, fixed.params, V.fixed, 
        Gamma=NULL, W=NULL, extra=FALSE, DEBUG=FALSE)
}
\arguments{
\item{mxFit}{An mxModel instance after it has been run}
\item{fixed.params}{Parameter index of the fixed but uncertain parameters 
    of the model. The }
}
\value{
    \item{error.variance}{The calculated value of the error variance estimate.}
}
\seealso{
\code{\link{get.stderr.corrected}}, \code{\link{guillen}}.
}
\examples{
    data(guillen)
    # Observed variance equals 3.61
    (obs.variance <- guillen[1])

    # Error variance for the SCF variable. The sample size (223),
    #   reliability (0.76), and standard error (0.04) of the reliability
    #   are taken directly or calculated indirectly from 
    # Guillen et al. (2009)
    get.err.var(obs.variance, 223, 0.76, 0.04^2)

    # If the SCF variable had a kurtosis of 6 (corresponding to a 
    #  t-distribution with 5 df), the error variance estimate
    #  would not change, but the variance of this estimate would increase:
    get.err.var(obs.variance, 223, 0.76, 0.04^2, excess.kurtosis = 6)
}
\keyword{get.err.var}
\references{
    Oberski, D. and A. Satorra (2010). How to take the variability of measurement
        error variances into account. http://daob.org/
}
\author{Daniel Oberski \email{daniel.oberski@upf.edu}}
