library(OpenMxExtras)
library(mvtnorm)

 # measurement error variances
psi <- c(1/3, 1/3)
         
# Variance matrix of the measurement error variances
sd.psi <- c(.1, .1)
S.psi <- diag(sd.psi^2)

# take the parameter estimates as true values
truevals <- c(1/2, 1, 3/4)

# model-implied matrix; take as true covmat
Sigma <- matrix(c(4/3, 1/2, 1/2, 4/3), 2, 2)
colnames(Sigma) <- rownames(Sigma) <- c('X', 'Y')

S.star <- Sigma
nobs <- 1500
source('R/montecarlo_small_model.r')
MeasErrFit <- mxRun(MeasErrModel)


# correct std errs
fixed.params <- c(get.par.index('S', 1, 1, MeasErrFit),
                  get.par.index('S', 2, 2, MeasErrFit))
V <- get.var.corrected(MeasErrFit, fixed.params, S.psi)

se.cor <- sqrt(diag(V$corrected))
se.uco <- sqrt(diag(V$uncorrected))

print(round(cbind(truevals, MeasErrFit@output$standardErrors, se.uco,se.cor), 2))


nobs <- 1500  # number of observations in simulated data set
nsim <-  700  # number of simulations
psi.orig <- psi # save original PSI values
# results of corrected matrix are in different order, correct order:
mu <- rep(0, NCOL(S.star)) # zero mean vector -> ignored in analysis
npar <- 3

var.psi.scales <- seq(0, 3, by=.25)
results <- list()

simulate.for.condition <- function(j) {

# whether true values were in resp. 95% CI
cis <- matrix(NA, nsim, npar)
cis.uco <- matrix(NA, nsim, npar)
cis.cor <- matrix(NA, nsim, npar)

# the calculated standard errors
ses <- matrix(NA, nsim, npar)
ses.uco <- matrix(NA, nsim, npar)
ses.cor <- matrix(NA, nsim, npar)

# The estimates
ests <- matrix(NA, nsim, npar)

# measurement error variances
psi <-c(1/3, 1/3)
  
simit <- function(it) { 
  X <- rmvnorm(nobs, mu, Sigma) # draw random sample from MVN with pop cov matrix
  colnames(X) <- colnames(Sigma) # rename variables
  S.star <- OpenMxExtras:::cov.ml(X) # Get ML cov matrix (uses n not n-1)
  # draw random estimate of PSI, with varied uncertainty
  S.psi <- var.psi.scales[j] * diag(sd.psi^2)
  psi <- as.numeric(rmvnorm(1, psi.orig, S.psi))
  
  source('R/montecarlo_small_model.r') # redefine model with new PSI and observed covmat

  e <- tryCatch(MeasErrFit <- mxRun(MeasErrModel), # re-estimate model w/new data and PSI
           error = function(e) e)
  if(any(class(e) %in% c("error", 'simpleError'))) {
    print("Error in optimization.")
    next
  }
  if(!all(MeasErrFit@output$estimate[c(2,3)]>0.0)) {
    print("Inadmissible solution: ")
    print(MeasErrFit@output$estimate)
    next
  }
  if(!MeasErrFit@output$status[[1]] %in% c(0,1)){
    print("Non 0 or 1 exit code")
    next
  }
  #print("Getting results..")
  
  # get the corrected se's
  e <- tryCatch(V <- get.var.corrected(MeasErrFit, fixed.params, S.psi),
           error = function(e) e)
  if(any(class(e) %in% c("error", 'simpleError'))) {
    print(paste("Could not get corrected se's: ", e))
    next
  }

  se.cor <- sqrt(diag(V$corrected))
  ses.cor[it,] <- se.cor # save to result matrix
  
  se.uco <- sqrt(diag(V$uncorrected)) # model-implied uncorrected se's
  ses.uco[it,] <- se.uco# save to result matrix
  
  se <-  MeasErrFit@output$standardErrors # numerical hessian-based uncorrected se's
  ses[it,] <- se# save to result matrix
  
  est <- MeasErrFit@output$estimate # model estimates for this draw
  ests[it,] <- est# save to result matrix

  # calculate whether true values are in 95% CI; save to result matrices
  cis[it,] <- (truevals > (est - 1.96*se)) & (truevals < (est + 1.96*se))
  cis.uco[it,] <- (truevals > (est - 1.96*se.uco)) & (truevals < (est + 1.96*se.uco))
  cis.cor[it,] <- (truevals > (est - 1.96*se.cor)) & (truevals < (est + 1.96*se.cor))
}
psi <- psi.orig

print(summary(ests))

#outlier.inds <- unlist(unique(outlier(1:10, 5)))
#print(paste("Found", length(outlier.inds),
#            "outliers (> 5 interquartile ranges):"))
#print(outlier.inds)

ests <- na.omit(ests)
ses <- na.omit(ses)
ses.uco <- na.omit(ses.uco)
ses.cor <- na.omit(ses.cor)
cis <- na.omit(cis)
cis.uco <- na.omit(cis.uco)
cis.cor <- na.omit(cis.cor)


print(sd.psi*sqrt(var.psi.scales[j]))
print(round(
results[[j]] <- cbind(avgest=apply(ests, 2, mean),
      relbias=(apply(ests, 2, mean) - truevals)/truevals,
      sd.est=apply(ests, 2, sd),
      avg.se=apply(ses, 2, mean),
      avg.se.uco=apply(ses.uco, 2, mean),
      avg.se.cor=apply(ses.cor, 2, mean),
      se.bias=apply(ses, 2, mean) - apply(ests, 2, sd), 
      se.uco.bias=apply(ses.uco, 2, mean) - apply(ests, 2, sd), 
      se.cor.bias=apply(ses.cor, 2, mean) - apply(ests, 2, sd),
      se.cor.rel.bias=(apply(ses.cor, 2, mean) - apply(ests, 2, sd))/truevals, 
      cis.95=apply(cis, 2, mean),
      cis.95.uco=apply(cis.uco, 2, mean),
      cis.95.cor=apply(cis.cor, 2, mean)
      ), 3)
)

#write.table(results[[j]], file=sprintf("results/result%d.tab", j))
}

rel <- c(.75, .75)
x.var <- psi/(1 - rel)
psi.sd <- c(.1, .1)
tab <- psi
for(sca in var.psi.scales)
  tab <- rbind(tab, sd.psi*sca)

Sig <- Sigma

relsds <- function(errvarsds) {
  apply(errvarsds, 1, function(row) {
    ei <- rmvnorm(nsim, psi, diag(row^2))
    
    x.vars <- t(mapply(function(i) {
      X <- rmvnorm(1500, rep(0, ncol(Sig)), Sig)
      S <- cov(X)
      colnames(S) <- rownames(S) <- colnames(Sig)
      diag(S)
    }, 1:nsim))
    
    rr <- t(mapply(function(i)
                   sqrt((x.vars[i,] - ei[i,])/x.vars[i,]), 1:nsim))
    
    apply(rr, 2, function(x) sd(x, na.rm=T))
  })
}

relsd <- relsds(tab[2:nrow(tab),])
colnames(relsd) <- var.psi.scales

library(xtable)
xtable(rtab <- cbind(t(tab)[,1:2], sqrt(rel), round(relsd, 2)))

